'*******************************************************************************
'Copyright 2002 Seapine Software, Inc. All rights reserved.
'
'IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
'in consideration of your agreement to the following terms, and your use, installation,
'modification or redistribution of this Software constitutes acceptance of these terms.
'If you do not agree with these terms, please do not use, install, modify or redistribute
'this Software.
'
'In consideration of your agreement to abide by the following terms, and subject to these
'terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
'this original Seapine software (the Software), to use, reproduce, modify and redistribute
'the Software, with or without modifications, in source and/or binary forms; provided that
'if you redistribute the Software in its entirety and without modifications, you must retain
'this notice and the following text and disclaimers in all such redistributions of the Software.
'Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
'or promote products derived from the Software without specific prior written permission from
'Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
'implied, are granted by Seapine herein, including but not limited to any patent rights that
'may be infringed by your derivative works or by other works in which the Software may be
'incorporated.
'
'The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
'EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
'MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
'OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS. 
'
'IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
'DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
'LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
'REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
'UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
'IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'******************************************************************************
Imports System
Imports System.Reflection
Imports System.Collections

Module EscalationExample

   Class CEscalationExample
      ' Private data members
      Private Const escalationPriority = "Immediate"
      Private cookie As Long
      Private soapServer As ttsoapcgi
      Private comparisonDate As DateTime ' value used as part of determining if a defect should be escalated

      '******************************************************************************
      ' Main
      '******
      Shared Sub Main()
         ' Use the argument verifier class to get and verify the command line arguments
         Dim arguments As CArguments
         arguments = New CArguments(Command().Split("-"))
         ' Create a new instance of our class
         Dim example = New CEscalationExample

         If (arguments.valid) Then
            Try
               ' Verify that the database specified on the command line exists
               If example.VerifyDatabase(arguments.database) Then
                  ' Connect to the TestTrack Pro SOAP CGI
                  example.Connect(arguments.database, arguments.username, arguments.password)
                  ' Verify that we can edit defects and that the priority to escalate to exists
                  If example.CanEditDefects() Then
                     If example.DoesEscalationPriorityExist() Then
                        example.EscalateDefects()
                     Else
                        Console.WriteLine("The priority, {0}, is not present on the database. Unable to escalate defects.", arguments.database)
                     End If
                  Else
                     Console.WriteLine("You do not have permission to edit defects.")
                  End If
               Else
                  Console.WriteLine("Error: The database ""{0}"" doesn't exist.", arguments.database)
               End If

            Catch e As System.Exception
               Console.WriteLine("Error: {0}", e.Message)

            Finally
               ' Ensure that we are disconnected
               example.Disconnect()
            End Try

         Else
            example.ShowUsage()
         End If
      End Sub

      '******************************************************************************
      '	Constructor - Create new ttsoapcgi and comparison date objects
      '**********
      Public Sub New()
         soapServer = New ttsoapcgi
         comparisonDate = New DateTime(DateTime.Now.AddMonths(-1).Ticks)
      End Sub

      '******************************************************************************
      ' ShowUsage - Show what arguments are used to invoke this program
      '*********
      Sub ShowUsage()
         Dim myModule As System.Reflection.Module
         myModule = System.Reflection.Assembly.GetExecutingAssembly().GetModules()(0)

         Console.WriteLine("Usage: {0} -u username [-p password] -d database", myModule.Name)
         Console.WriteLine("Example 1: {0} -u admin -p secret -d TestDB", myModule.Name)
         Console.WriteLine("Example 2: {0} -u admin -d Sample Database", myModule.Name)
      End Sub

      '******************************************************************************
      ' Connect - Connects to a the given database using the supplied user name and
      '           password
      '********
      Public Sub Connect(ByVal database As String, ByVal username As String, ByVal password As String)
         Try
            cookie = soapServer.DatabaseLogon(database, username, password)
         Catch e As System.Exception
            Console.WriteLine("Error: {0}", e.Message)
         End Try
      End Sub

      '******************************************************************************
      ' Disconnect - If we're currently connected to a database, disconnect
      '*********
      Public Sub Disconnect()
         If Not cookie = 0 Then
            Try
               soapServer.DatabaseLogoff(cookie)
            Catch e As System.Exception
               Console.WriteLine("Error: {0}", e.Message)
            End Try
         End If
      End Sub

      '******************************************************************************
      ' VerifyDatabase - Verifies that the database specified is on the server
      '**********
      Public Function VerifyDatabase(ByVal dbName As String) As Boolean
         Dim result = False
         Dim databaseList() As CDatabase
         databaseList = soapServer.getDatabaseList()
         Dim db As CDatabase

         For Each db In databaseList
            If db.name = dbName Then
               result = True
               Exit For
            End If
         Next db

         VerifyDatabase = result
      End Function

      '******************************************************************************
      '	CanEditDefects - Verifies that the logged in user can edit defects
      '***********
      Public Function CanEditDefects() As Boolean
         Dim bResult = False
         Dim tables() As CDatabaseTable
         tables = soapServer.getTableList(cookie)
         Dim table As CDatabaseTable

         For Each table In tables
            If table.name = "Defect" Then
               bResult = True
               Exit For
            End If
         Next table

         CanEditDefects = bResult
      End Function

      '******************************************************************************
      ' DoesEscalationPriorityExit - Verifies that escalationPriority exists as
      '                              a priority in the database
      '*********
      Public Function DoesEscalationPriorityExist() As Boolean
         Dim bResult = False
         Dim fieldValues() As CFieldValue
         fieldValues = soapServer.getDropdownFieldValuesForTable(cookie, "Defect", "Priority")
         Dim fieldValue As CFieldValue

         For Each fieldValue In fieldValues
            If fieldValue.value = escalationPriority Then
               bResult = True
               Exit For
            End If
         Next fieldValue

         DoesEscalationPriorityExist = bResult
      End Function

      '******************************************************************************
      '	EscalateDefects - Escalates all defects to the "Immediate" priority in the
      '                    current database that meet all of these criteria:
      '                    1. State contains "Open"
      '                    2. Defect is at least a month old, or older
      '                   3. Defect isn't marked with the "Immediate" priority
      '*************
      Public Sub EscalateDefects()
         Dim list As CRecordListSoap
         list = GetDefectList()
         Dim record As CRecordRowSoap

         If list.records.Length > 0 Then
            For Each record In list.records
               ' See if this defect matches our search criteria
               If record.row(2).value.StartsWith("Open") _
                  And IsDateAtLeastOneMonthOld(record.row(3).value) _
                  And Not record.row(4).value = "Immediate" Then

                  Try
                     ' Note: It is possible that the defect hasn't yet been assigned
                     '       a defect number, use 0 in this case
                     Dim defectNumber As Long

                     If record.row(0).value = "-" Then
                        defectNumber = 0
                     Else
                        defectNumber = Convert.ToInt64(record.row(0).value)
                     End If

                     Dim defect = soapServer.editDefect(cookie, defectNumber, record.row(1).value, False)
                     defect.priority = "Immediate"
                     soapServer.saveDefect(cookie, defect)

                  Catch e As System.Exception
                     Console.WriteLine("Error occurred while attempting to edit defect number ""{0}"" with summary ""{1}""", _
                          record.row(0).value, _
                          record.row(1).value)
                     Console.WriteLine("Error message: ""{0}""", e.Message)
                  End Try
               End If
            Next record
         End If
      End Sub

      '******************************************************************************
      ' GetDefectList - Returns an array of defects on the current datbase logged
      '                 into.The columns in the returned list will be defect number,
      '                 summary, status, date entered, and priority
      '*************
      Public Function GetDefectList() As CRecordListSoap
         ' Create a CTableColumn array
         Dim columnsToReturn(5) As CTableColumn
         Dim i As Integer

         For i = 0 To 4
            columnsToReturn(i) = New CTableColumn
         Next i
         columnsToReturn(0).name = "Number"
         columnsToReturn(1).name = "Summary"
         columnsToReturn(2).name = "Status"
         columnsToReturn(3).name = "Date Entered"
         columnsToReturn(4).name = "Priority"

         GetDefectList = soapServer.getRecordListForTable(cookie, "Defect", "", columnsToReturn)
      End Function

      '******************************************************************************
      ' IsDateAtLeastOneMonthOld - Returns true if the given string representation
      ' of a date is a month or more old
      '**************
      Public Function IsDateAtLeastOneMonthOld(ByRef [date] As String) As Boolean
         Dim result = False

         If [date].Length = 0 Then
            result = True
         Else
            If Convert.ToDateTime([date]) <= comparisonDate Then
               result = True
            End If
         End If

         IsDateAtLeastOneMonthOld = result
      End Function

   End Class

#Region "CArguments Utility Class"
   '##########################################################################
   '#	CArguments	- Private utility class to get and verify command line args.
   '##########################################################################
   Private Class CArguments
      Public ReadOnly username As String
      Public ReadOnly password As String
      Public ReadOnly database As String
      Public ReadOnly valid As Boolean    'if true then the values above are valid

      Public Sub New(ByRef args() As String)
         username = ""
         password = ""
         database = ""
         valid = False

         If args.Length = 4 Then
            If args(1).StartsWith("u") And args(2).StartsWith("p") And args(3).StartsWith("d") Then
               username = args(1).Substring(2, args(1).Length - 3)
               password = args(2).Substring(2, args(1).Length - 3)
               database = args(3).Substring(2)
            End If
         ElseIf args.Length = 3 Then
            If args(1).StartsWith("u") And args(2).StartsWith("d") Then
               username = args(1).Substring(2, args(1).Length - 3)
               database = args(2).Substring(2)
            End If
         Else
            Console.WriteLine("Error: Invalid command line options.")
            Exit Sub
         End If

         If Not username.Length = 0 And Not database.Length = 0 Then
            valid = True
         ElseIf username.Length = 0 And database.Length = 0 Then
            Console.WriteLine("Error: User name and database name are blank.")
         ElseIf username.Length = 0 Then
            Console.WriteLine("Error: User name is blank.")
         ElseIf database.Length = 0 Then
            Console.WriteLine("Error: Database name is blank.")
         ElseIf args(0) = "" Then
            Console.WriteLine("Error: Invalid number and/or type of arguments")
         End If
      End Sub

   End Class
#End Region

End Module
